@@ -1,23 +1,28 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  * Copyright (C) 2006 OpenedHand Ltd
+ * Copyright (C) 2009 Intel Corporation
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of version 2 of the GNU Lesser General Public License as published
  * by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
+ * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  * details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * along with this library; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Author: Ross Burton <ross@openedhand.com>
+ * Author: Ross Burton <ross@linux.intel.com>
  */
 
+/* Define this so we get the full API defined internally */
+#define EDS_WITH_DBUS_ONLY_API
+
 #include <config.h>
 #include <unistd.h>
 #include <string.h>
@@ -1901,6 +1906,22 @@ e_book_cancel (EBook *book, GError **error)
   return org_gnome_evolution_dataserver_addressbook_Book_cancel_operation (book->priv->proxy, error);
 }
 
+/**
+ * e_book_cancel_async_op:
+ * Similar to above e_book_cancel function, only cancels last, still running,
+ * asynchronous operation.
+ **/
+gboolean
+e_book_cancel_async_op (EBook *book, GError **error)
+{
+  e_return_error_if_fail (E_IS_BOOK (book), E_BOOK_ERROR_INVALID_ARG);
+  e_return_error_if_fail (book->priv->proxy, E_BOOK_ERROR_REPOSITORY_OFFLINE);
+
+  g_debug ("TODO: implement e_book_cancel_async_op");
+
+  return org_gnome_evolution_dataserver_addressbook_Book_cancel_operation (book->priv->proxy, error);
+}
+
 /**
  * e_book_get_uri:
  * @book: an #EBook
