@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* e-vcard.c
  *
- * Copyright (C) 2003 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -280,7 +280,7 @@ read_attribute_value (EVCardAttribute *attr, const char **p, gboolean quoted_pri
 			lp = g_utf8_next_char(lp);
 		}
 		else if ((*lp == ';') ||
-			 (*lp == ',' && !strcmp (attr->name, "CATEGORIES"))) {
+			 (*lp == ',' && !g_ascii_strcasecmp (attr->name, "CATEGORIES"))) {
 			WRITE_CHUNK;
 			e_vcard_attribute_add_value (attr, str->str);
 			g_string_assign (str, "");
@@ -373,7 +373,7 @@ read_attribute_params (EVCardAttribute *attr, const char **p, gboolean *quoted_p
 				}
 
 				if (param
-				    && !strcmp (param->name, "ENCODING")
+				    && !g_ascii_strcasecmp (param->name, "encoding")
 				    && !g_ascii_strcasecmp (param->values->data, "quoted-printable")) {
 					*quoted_printable = TRUE;
 					e_vcard_attribute_param_free (param);
@@ -559,25 +559,25 @@ parse (EVCard *evc, const char *str)
 		return;
 	
 	attr = read_attribute (&str);
-	if (!attr || attr->group || strcmp (attr->name, "BEGIN")) {
+	if (!attr || attr->group || g_ascii_strcasecmp (attr->name, "BEGIN")) {
 		g_warning ("vcard began without a BEGIN:VCARD\n");
 	}
-	if (attr && !strcmp (attr->name, "BEGIN"))
+	if (attr && !g_ascii_strcasecmp (attr->name, "BEGIN"))
 		e_vcard_attribute_free (attr);
 	else if (attr)
 		e_vcard_add_attribute (evc, attr);
 	
 	while ((attr = read_attribute (&str))) {
-		if (strcmp (attr->name, "END") != 0)
+		if (g_ascii_strcasecmp (attr->name, "END") != 0)
 			e_vcard_add_attribute (evc, attr);
 		else
 			break;
 	}
 
-	if (!attr || attr->group || strcmp (attr->name, "END"))
+	if (!attr || attr->group || g_ascii_strcasecmp (attr->name, "END"))
 		g_warning ("vcard ended without END:VCARD\n");
 	
-	if (attr && !strcmp (attr->name, "END"))
+	if (attr && !g_ascii_strcasecmp (attr->name, "END"))
 		e_vcard_attribute_free (attr);
 
 	evc->priv->attributes = g_list_reverse (evc->priv->attributes);
@@ -745,7 +745,7 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		EVCardAttribute *attr = l->data;
 		GString *attr_str;
 
-		if (!strcmp (attr->name, "VERSION"))
+		if (!g_ascii_strcasecmp (attr->name, "VERSION"))
 			continue;
 
 		attr_str = g_string_new ("");
@@ -806,7 +806,7 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 				/* XXX toshok - i hate you, rfc 2426.
 				   why doesn't CATEGORIES use a ; like
 				   a normal list attribute? */
-				if (!strcmp (attr->name, "CATEGORIES"))
+				if (!g_ascii_strcasecmp (attr->name, "CATEGORIES"))
 					g_string_append_c (attr_str, ',');
 				else
 					g_string_append_c (attr_str, ';');
@@ -1365,7 +1365,7 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
 
 	/* we handle our special encoding stuff here */
 
-	if (!strcmp (param->name, EVC_ENCODING)) {
+	if (!g_ascii_strcasecmp (param->name, EVC_ENCODING)) {
 		if (attr->encoding_set) {
 			g_warning ("ENCODING specified twice");
 			return;
@@ -1597,7 +1597,7 @@ e_vcard_get_attribute (EVCard     *vcard,
                 EVCardAttribute *attr;
 		
                 attr = (EVCardAttribute *) l->data;
-                if (strcmp (attr->name, name) == 0)
+                if (g_ascii_strcasecmp (attr->name, name) == 0)
                         return attr;
         }
 
