@@ -1,25 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* Evolution calendar - generic backend class
  *
- * Copyright (C) 2000 Ximian, Inc.
- * Copyright (C) 2000 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * Authors: Federico Mena-Quintero <federico@ximian.com>
  *          JP Rosevear <jpr@ximian.com>
- *          Rodrigo Moya <rodrigo@ximian.com>    
+ *          Rodrigo Moya <rodrigo@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -31,6 +30,8 @@
 
 
 
+G_DEFINE_TYPE (ECalBackend, e_cal_backend, G_TYPE_OBJECT);
+
 /* Private part of the CalBackend structure */
 struct _ECalBackendPrivate {
 	/* The source for this backend */
@@ -41,7 +42,7 @@ struct _ECalBackendPrivate {
 
 	/* The kind of components for this backend */
 	icalcomponent_kind kind;
-	
+
 	/* List of Cal objects */
 	GMutex *clients_mutex;
 	GList *clients;
@@ -51,10 +52,18 @@ struct _ECalBackendPrivate {
 
 	/* ECalBackend to pass notifications on to */
 	ECalBackend *notification_proxy;
+
+	/* used when notifying clients about progress of some operation,
+	 * we do not send multiple notifications with the same percent
+	 * value */
+	int last_percent_notified;
 };
 
 #define E_CAL_BACKEND_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), E_TYPE_CAL_BACKEND, ECalBackendPrivate))
 
+/* For convenience */
+#define CLASS(backend) (E_CAL_BACKEND_GET_CLASS(backend))
+
 /* Property IDs */
 enum props {
 	PROP_0,
@@ -72,55 +81,19 @@ enum {
 };
 static guint e_cal_backend_signals[LAST_SIGNAL];
 
-static void e_cal_backend_class_init (ECalBackendClass *class);
-static void e_cal_backend_init (ECalBackend *backend);
 static void e_cal_backend_finalize (GObject *object);
 
-#define CLASS(backend) (E_CAL_BACKEND_CLASS (G_OBJECT_GET_CLASS (backend)))
-
-static GObjectClass *parent_class;
-
 
 
-/**
- * e_cal_backend_get_type:
- *
- * Registers the #ECalBackend class if necessary, and returns the type ID
- * associated to it.
- *
- * Return value: The type ID of the #ECalBackend class.
- **/
-GType
-e_cal_backend_get_type (void)
-{
-	static GType e_cal_backend_type = 0;
-
-	if (!e_cal_backend_type) {
-		static GTypeInfo info = {
-                        sizeof (ECalBackendClass),
-                        (GBaseInitFunc) NULL,
-                        (GBaseFinalizeFunc) NULL,
-                        (GClassInitFunc) e_cal_backend_class_init,
-                        NULL, NULL,
-                        sizeof (ECalBackend),
-                        0,
-                        (GInstanceInitFunc) e_cal_backend_init,
-                };
-		e_cal_backend_type = g_type_register_static (G_TYPE_OBJECT, "ECalBackend", &info, 0);
-	}
-
-	return e_cal_backend_type;
-}
-
 static void
 e_cal_backend_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
 {
 	ECalBackend *backend;
 	ECalBackendPrivate *priv;
-	
+
 	backend = E_CAL_BACKEND (object);
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-	
+	priv = backend->priv;
+
 	switch (property_id) {
 	case PROP_SOURCE:
 		{
@@ -162,9 +135,9 @@ e_cal_backend_get_property (GObject *object, guint property_id, GValue *value, G
 {
  	ECalBackend *backend;
 	ECalBackendPrivate *priv;
-	
+
 	backend = E_CAL_BACKEND (object);
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	switch (property_id) {
 	case PROP_SOURCE:
@@ -184,37 +157,35 @@ e_cal_backend_get_property (GObject *object, guint property_id, GValue *value, G
 
 /* Class initialization function for the calendar backend */
 static void
-e_cal_backend_class_init (ECalBackendClass *klass)
+e_cal_backend_class_init (ECalBackendClass *class)
 {
-	GObjectClass *parent_class, *object_class;
+	GObjectClass *object_class;
 
-	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
-
-	object_class = (GObjectClass *) klass;
+	object_class = (GObjectClass *) class;
 
 	object_class->set_property = e_cal_backend_set_property;
 	object_class->get_property = e_cal_backend_get_property;
 	object_class->finalize = e_cal_backend_finalize;
 
-	g_object_class_install_property (object_class, PROP_SOURCE, 
+	g_object_class_install_property (object_class, PROP_SOURCE,
 					 g_param_spec_object ("source", NULL, NULL, E_TYPE_SOURCE,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE
 							      | G_PARAM_CONSTRUCT_ONLY));
 
-	g_object_class_install_property (object_class, PROP_URI, 
+	g_object_class_install_property (object_class, PROP_URI,
 					 g_param_spec_string ("uri", NULL, NULL, "",
 							      G_PARAM_READABLE | G_PARAM_WRITABLE
 							      | G_PARAM_CONSTRUCT_ONLY));
 
-	g_object_class_install_property (object_class, PROP_KIND, 
-					 g_param_spec_ulong ("kind", NULL, NULL, 
-							     ICAL_NO_COMPONENT, ICAL_XLICMIMEPART_COMPONENT, 
+	g_object_class_install_property (object_class, PROP_KIND,
+					 g_param_spec_ulong ("kind", NULL, NULL,
+							     ICAL_NO_COMPONENT, ICAL_XLICMIMEPART_COMPONENT,
 							     ICAL_NO_COMPONENT,
 							     G_PARAM_READABLE | G_PARAM_WRITABLE
-							     | G_PARAM_CONSTRUCT_ONLY));	
+							     | G_PARAM_CONSTRUCT_ONLY));
 	e_cal_backend_signals[LAST_CLIENT_GONE] =
 		g_signal_new ("last_client_gone",
-			      G_TYPE_FROM_CLASS (klass),
+			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (ECalBackendClass, last_client_gone),
 			      NULL, NULL,
@@ -222,7 +193,7 @@ e_cal_backend_class_init (ECalBackendClass *klass)
 			      G_TYPE_NONE, 0);
 	e_cal_backend_signals[OPENED] =
 		g_signal_new ("opened",
-			      G_TYPE_FROM_CLASS (klass),
+			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (ECalBackendClass, opened),
 			      NULL, NULL,
@@ -231,7 +202,7 @@ e_cal_backend_class_init (ECalBackendClass *klass)
 			      G_TYPE_INT);
 	e_cal_backend_signals[REMOVED] =
 		g_signal_new ("removed",
-			      G_TYPE_FROM_CLASS (klass),
+			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (ECalBackendClass, removed),
 			      NULL, NULL,
@@ -239,59 +210,61 @@ e_cal_backend_class_init (ECalBackendClass *klass)
 			      G_TYPE_NONE, 1,
 			      G_TYPE_INT);
 
-	klass->last_client_gone = NULL;
-	klass->opened = NULL;
-	klass->obj_updated = NULL;
+	class->last_client_gone = NULL;
+	class->opened = NULL;
+	class->obj_updated = NULL;
 
-	klass->get_cal_address = NULL;
-	klass->get_alarm_email_address = NULL;
-	klass->get_static_capabilities = NULL;
-	klass->open = NULL;
-	klass->is_loaded = NULL;
-	klass->is_read_only = NULL;
-	klass->start_query = NULL;
-	klass->get_mode = NULL;
-	klass->set_mode = NULL;	
-	klass->get_object = NULL;
-	klass->get_default_object = NULL;
-	klass->get_object_list = NULL;
-	klass->get_free_busy = NULL;
-	klass->get_changes = NULL;
-	klass->discard_alarm = NULL;
-	klass->create_object = NULL;
-	klass->modify_object = NULL;
-	klass->remove_object = NULL;
-	klass->receive_objects = NULL;
-	klass->send_objects = NULL;
-	klass->get_timezone = NULL;
-	klass->add_timezone = NULL;
-	klass->set_default_timezone = NULL;
+	class->get_cal_address = NULL;
+	class->get_alarm_email_address = NULL;
+	class->get_static_capabilities = NULL;
+	class->open = NULL;
+	class->is_loaded = NULL;
+	class->is_read_only = NULL;
+	class->start_query = NULL;
+	class->get_mode = NULL;
+	class->set_mode = NULL;
+	class->get_object = NULL;
+	class->get_default_object = NULL;
+	class->get_object_list = NULL;
+	class->get_free_busy = NULL;
+	class->get_changes = NULL;
+	class->discard_alarm = NULL;
+	class->create_object = NULL;
+	class->modify_object = NULL;
+	class->remove_object = NULL;
+	class->receive_objects = NULL;
+	class->send_objects = NULL;
+	class->get_timezone = NULL;
+	class->add_timezone = NULL;
+	class->set_default_timezone = NULL;
 
-	g_type_class_add_private (klass, sizeof (ECalBackendPrivate));
+	g_type_class_add_private (class, sizeof (ECalBackendPrivate));
 }
 
 /* Object initialization func for the calendar backend */
-void
+static void
 e_cal_backend_init (ECalBackend *backend)
 {
 	ECalBackendPrivate *priv;
 
 	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	backend->priv = priv;
 
 	priv->clients = NULL;
 	priv->clients_mutex = g_mutex_new ();
+	priv->last_percent_notified = 0;
 
 	priv->queries = e_list_new((EListCopyFunc) g_object_ref, (EListFreeFunc) g_object_unref, NULL);
 	priv->queries_mutex = g_mutex_new ();
 }
 
-void
+static void
 e_cal_backend_finalize (GObject *object)
 {
 	ECalBackend *backend = (ECalBackend *)object;
 	ECalBackendPrivate *priv;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	g_assert (priv->clients == NULL);
 
@@ -300,9 +273,10 @@ e_cal_backend_finalize (GObject *object)
 	g_mutex_free (priv->clients_mutex);
 	g_mutex_free (priv->queries_mutex);
 
-	g_free (priv);
+	g_free (priv->uri);
+	g_object_unref (priv->source);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (e_cal_backend_parent_class)->finalize (object);
 }
 
 
@@ -319,12 +293,12 @@ ESource *
 e_cal_backend_get_source (ECalBackend *backend)
 {
 	ECalBackendPrivate *priv;
-	
+
 	g_return_val_if_fail (backend != NULL, NULL);
 	g_return_val_if_fail (E_IS_CAL_BACKEND (backend), NULL);
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-	
+	priv = backend->priv;
+
 	return priv->source;
 }
 
@@ -341,12 +315,12 @@ const char *
 e_cal_backend_get_uri (ECalBackend *backend)
 {
 	ECalBackendPrivate *priv;
-	
+
 	g_return_val_if_fail (backend != NULL, NULL);
 	g_return_val_if_fail (E_IS_CAL_BACKEND (backend), NULL);
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-	
+	priv = backend->priv;
+
 	return priv->uri;
 }
 
@@ -362,12 +336,12 @@ icalcomponent_kind
 e_cal_backend_get_kind (ECalBackend *backend)
 {
 	ECalBackendPrivate *priv;
-	
+
 	g_return_val_if_fail (backend != NULL, ICAL_NO_COMPONENT);
 	g_return_val_if_fail (E_IS_CAL_BACKEND (backend), ICAL_NO_COMPONENT);
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-	
+	priv = backend->priv;
+
 	return priv->kind;
 }
 
@@ -389,20 +363,22 @@ void
 e_cal_backend_add_client (ECalBackend *backend, EDataCal *cal)
 {
 	ECalBackendPrivate *priv;
-	
+
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
 	g_return_if_fail (cal != NULL);
 	g_return_if_fail (E_IS_DATA_CAL (cal));
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-	
-	/* FIXME: Implement this? */
-/*	bonobo_object_set_immortal (BONOBO_OBJECT (cal), TRUE);
+	priv = backend->priv;
+
+	/* TODO: Implement this? */
+#if 0
+	bonobo_object_set_immortal (BONOBO_OBJECT (cal), TRUE);
 
 	g_object_weak_ref (G_OBJECT (cal), cal_destroy_cb, backend);
 
-	ORBit_small_listen_for_broken (e_data_cal_get_listener (cal), G_CALLBACK (listener_died_cb), cal);*/
+	ORBit_small_listen_for_broken (e_data_cal_get_listener (cal), G_CALLBACK (listener_died_cb), cal);
+#endif
 
 	g_mutex_lock (priv->clients_mutex);
 	priv->clients = g_list_append (priv->clients, cal);
@@ -420,7 +396,7 @@ void
 e_cal_backend_remove_client (ECalBackend *backend, EDataCal *cal)
 {
 	ECalBackendPrivate *priv;
-	
+
 	/* XXX this needs a bit more thinking wrt the mutex - we
 	   should be holding it when we check to see if clients is
 	   NULL */
@@ -429,7 +405,7 @@ e_cal_backend_remove_client (ECalBackend *backend, EDataCal *cal)
 	g_return_if_fail (cal != NULL);
 	g_return_if_fail (E_IS_DATA_CAL (cal));
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	/* Disconnect */
 	g_mutex_lock (priv->clients_mutex);
@@ -455,18 +431,14 @@ e_cal_backend_remove_client (ECalBackend *backend, EDataCal *cal)
 void
 e_cal_backend_add_query (ECalBackend *backend, EDataCalView *query)
 {
-	ECalBackendPrivate *priv;
-
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-	
-	g_mutex_lock (priv->queries_mutex);
+	g_mutex_lock (backend->priv->queries_mutex);
 
-	e_list_append (priv->queries, query);
-	
-	g_mutex_unlock (priv->queries_mutex);
+	e_list_append (backend->priv->queries, query);
+
+	g_mutex_unlock (backend->priv->queries_mutex);
 }
 
 /**
@@ -480,16 +452,31 @@ e_cal_backend_add_query (ECalBackend *backend, EDataCalView *query)
 EList *
 e_cal_backend_get_queries (ECalBackend *backend)
 {
-	ECalBackendPrivate *priv;
-	
 	g_return_val_if_fail (backend != NULL, NULL);
 	g_return_val_if_fail (E_IS_CAL_BACKEND (backend), NULL);
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-	
-	return priv->queries;
+	return backend->priv->queries;
 }
 
+/**
+ * e_cal_backend_remove_query
+ * @backend: An #ECalBackend object.
+ * @query: An #EDataCalView object, previously added with @ref e_cal_backend_add_query.
+ *
+ * Removes query from the list of live queries for the backend.
+ **/
+void
+e_cal_backend_remove_query (ECalBackend *backend, EDataCalView *query)
+{
+	g_return_if_fail (backend != NULL);
+	g_return_if_fail (E_IS_CAL_BACKEND (backend));
+
+	g_mutex_lock (backend->priv->queries_mutex);
+
+	e_list_remove (backend->priv->queries, query);
+
+	g_mutex_unlock (backend->priv->queries_mutex);
+}
 
 /**
  * e_cal_backend_get_cal_address:
@@ -499,7 +486,7 @@ e_cal_backend_get_queries (ECalBackend *backend)
  * must already have an open calendar.
  **/
 void
-e_cal_backend_get_cal_address (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context)
+e_cal_backend_get_cal_address (ECalBackend *backend, EDataCal *cal, EServerMethodContext context)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -514,14 +501,26 @@ e_cal_backend_notify_readonly (ECalBackend *backend, gboolean read_only)
 	ECalBackendPrivate *priv;
 	GList *l;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	if (priv->notification_proxy) {
 		e_cal_backend_notify_readonly (priv->notification_proxy, read_only);
 		return;
 	}
 	for (l = priv->clients; l; l = l->next)
-		e_data_cal_notify_read_only (l->data, Success, read_only);
+		e_data_cal_notify_read_only (l->data, GNOME_Evolution_Calendar_Success, read_only);
+}
+
+void
+e_cal_backend_notify_cal_address (ECalBackend *backend, EServerMethodContext context, char *address)
+{
+	ECalBackendPrivate *priv;
+	GList *l;
+
+	priv = backend->priv;
+
+	for (l = priv->clients; l; l = l->next)
+		e_data_cal_notify_cal_address (l->data, context, GNOME_Evolution_Calendar_Success, address);
 }
 
 /**
@@ -532,7 +531,7 @@ e_cal_backend_notify_readonly (ECalBackend *backend, gboolean read_only)
  * Calls the get_alarm_email_address method on the given backend.
  */
 void
-e_cal_backend_get_alarm_email_address (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context)
+e_cal_backend_get_alarm_email_address (ECalBackend *backend, EDataCal *cal, EServerMethodContext context)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -549,7 +548,7 @@ e_cal_backend_get_alarm_email_address (ECalBackend *backend, EDataCal *cal, DBus
  * Calls the get_ldap_attribute method of the given backend.
  */
 void
-e_cal_backend_get_ldap_attribute (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context)
+e_cal_backend_get_ldap_attribute (ECalBackend *backend, EDataCal *cal, EServerMethodContext context)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -566,7 +565,7 @@ e_cal_backend_get_ldap_attribute (ECalBackend *backend, EDataCal *cal, DBusGMeth
  * Calls the get_static_capabilities method on the given backend.
  */
 void
-e_cal_backend_get_static_capabilities (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context)
+e_cal_backend_get_static_capabilities (ECalBackend *backend, EDataCal *cal, EServerMethodContext context)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -589,7 +588,7 @@ e_cal_backend_get_static_capabilities (ECalBackend *backend, EDataCal *cal, DBus
  * URI.
  */
 void
-e_cal_backend_open (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, gboolean only_if_exists,
+e_cal_backend_open (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, gboolean only_if_exists,
 		    const char *username, const char *password)
 {
 	g_return_if_fail (backend != NULL);
@@ -607,7 +606,7 @@ e_cal_backend_open (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *
  * Removes the calendar being accessed by the given backend.
  */
 void
-e_cal_backend_remove (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context)
+e_cal_backend_remove (ECalBackend *backend, EDataCal *cal, EServerMethodContext context)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -619,9 +618,9 @@ e_cal_backend_remove (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation
 /**
  * e_cal_backend_is_loaded:
  * @backend: A calendar backend.
- * 
+ *
  * Queries whether a calendar backend has been loaded yet.
- * 
+ *
  * Return value: TRUE if the backend has been loaded with data, FALSE
  * otherwise.
  */
@@ -664,7 +663,7 @@ e_cal_backend_is_read_only (ECalBackend *backend, EDataCal *cal)
  *
  * Starts a new live query on the given backend.
  */
-void 
+void
 e_cal_backend_start_query (ECalBackend *backend, EDataCalView *query)
 {
 	g_return_if_fail (backend != NULL);
@@ -676,10 +675,10 @@ e_cal_backend_start_query (ECalBackend *backend, EDataCalView *query)
 
 /**
  * e_cal_backend_get_mode:
- * @backend: A calendar backend. 
- * 
+ * @backend: A calendar backend.
+ *
  * Queries whether a calendar backend is connected remotely.
- * 
+ *
  * Return value: The current mode the calendar is in
  **/
 CalMode
@@ -701,8 +700,8 @@ e_cal_backend_get_mode (ECalBackend *backend)
  * e_cal_backend_set_mode:
  * @backend: A calendar backend
  * @mode: Mode to change to
- * 
- * Sets the mode of the calendar 
+ *
+ * Sets the mode of the calendar
  */
 void
 e_cal_backend_set_mode (ECalBackend *backend, CalMode mode)
@@ -722,7 +721,7 @@ e_cal_backend_set_mode (ECalBackend *backend, CalMode mode)
  * Calls the get_default_object method on the given backend.
  */
 void
-e_cal_backend_get_default_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context)
+e_cal_backend_get_default_object (ECalBackend *backend, EDataCal *cal, EServerMethodContext context)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -742,7 +741,7 @@ e_cal_backend_get_default_object (ECalBackend *backend, EDataCal *cal, DBusGMeth
  * identifier and its recurrence ID (if a recurrent appointment).
  */
 void
-e_cal_backend_get_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *uid, const char *rid)
+e_cal_backend_get_object (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *uid, const char *rid)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -757,11 +756,11 @@ e_cal_backend_get_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInvoca
  * @backend: A calendar backend.
  * @cal: An #EDataCal object.
  * @sexp: Expression to search for.
- * 
+ *
  * Calls the get_object_list method on the given backend.
  */
 void
-e_cal_backend_get_object_list (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *sexp)
+e_cal_backend_get_object_list (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *sexp)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -778,14 +777,15 @@ e_cal_backend_get_object_list (ECalBackend *backend, EDataCal *cal, DBusGMethodI
  * @rid: ID for the object's recurrence to get.
  *
  * Queries a calendar backend for attachments present in a calendar object based
- * on its unique identifier and its recurrence ID (if a recurrent appointment).+ */
+ * on its unique identifier and its recurrence ID (if a recurrent appointment).
+ */
 void
-e_cal_backend_get_attachment_list (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *uid, const char *rid)
+e_cal_backend_get_attachment_list (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *uid, const char *rid)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
 	g_return_if_fail (uid != NULL);
-	
+
 	g_assert (CLASS (backend)->get_object != NULL);
 	(* CLASS (backend)->get_attachment_list) (backend, cal, context, uid, rid);
 }
@@ -797,11 +797,11 @@ e_cal_backend_get_attachment_list (ECalBackend *backend, EDataCal *cal, DBusGMet
  * @users: List of users to get free/busy information for.
  * @start: Start time for query.
  * @end: End time for query.
- * 
+ *
  * Gets a free/busy object for the given time interval
  */
 void
-e_cal_backend_get_free_busy (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, GList *users, time_t start, time_t end)
+e_cal_backend_get_free_busy (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, GList *users, time_t start, time_t end)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -817,12 +817,12 @@ e_cal_backend_get_free_busy (ECalBackend *backend, EDataCal *cal, DBusGMethodInv
  * @backend: A calendar backend.
  * @cal: An #EDataCal object.
  * @change_id: A unique uid for the callers change list
- * 
+ *
  * Builds a sequence of objects and the type of change that occurred on them since
  * the last time the give change_id was seen
  */
 void
-e_cal_backend_get_changes (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *change_id) 
+e_cal_backend_get_changes (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *change_id)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -843,7 +843,7 @@ e_cal_backend_get_changes (ECalBackend *backend, EDataCal *cal, DBusGMethodInvoc
  * to do whatever is needed to really discard the alarm.
  */
 void
-e_cal_backend_discard_alarm (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *uid, const char *auid)
+e_cal_backend_discard_alarm (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *uid, const char *auid)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -863,7 +863,7 @@ e_cal_backend_discard_alarm (ECalBackend *backend, EDataCal *cal, DBusGMethodInv
  * Calls the create_object method on the given backend.
  */
 void
-e_cal_backend_create_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *calobj)
+e_cal_backend_create_object (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *calobj)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -885,7 +885,7 @@ e_cal_backend_create_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInv
  * Calls the modify_object method on the given backend.
  */
 void
-e_cal_backend_modify_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *calobj, CalObjModType mod)
+e_cal_backend_modify_object (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *calobj, CalObjModType mod)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -904,12 +904,12 @@ e_cal_backend_modify_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInv
  * @uid: Unique identifier of the object to remove.
  * @rid: A recurrence ID.
  * @mod: Type of removal.
- * 
+ *
  * Removes an object in a calendar backend.  The backend will notify all of its
  * clients about the change.
  */
 void
-e_cal_backend_remove_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *uid, const char *rid, CalObjModType mod)
+e_cal_backend_remove_object (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *uid, const char *rid, CalObjModType mod)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -928,7 +928,7 @@ e_cal_backend_remove_object (ECalBackend *backend, EDataCal *cal, DBusGMethodInv
  * Calls the receive_objects method on the given backend.
  */
 void
-e_cal_backend_receive_objects (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *calobj)
+e_cal_backend_receive_objects (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *calobj)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -947,7 +947,7 @@ e_cal_backend_receive_objects (ECalBackend *backend, EDataCal *cal, DBusGMethodI
  * Calls the send_objects method on the given backend.
  */
 void
-e_cal_backend_send_objects (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *calobj)
+e_cal_backend_send_objects (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *calobj)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -963,12 +963,12 @@ e_cal_backend_send_objects (ECalBackend *backend, EDataCal *cal, DBusGMethodInvo
  * @cal: An #EDataCal object.
  * @tzid: Unique identifier of a VTIMEZONE object. Note that this must not be
  * NULL.
- * 
+ *
  * Returns the icaltimezone* corresponding to the TZID, or NULL if the TZID
  * can't be found.
  */
 void
-e_cal_backend_get_timezone (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *tzid)
+e_cal_backend_get_timezone (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *tzid)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
@@ -979,22 +979,45 @@ e_cal_backend_get_timezone (ECalBackend *backend, EDataCal *cal, DBusGMethodInvo
 }
 
 /**
+ * e_cal_backend_set_default_zone:
+ * @backend: A calendar backend.
+ * @cal: An #EDataCal object.
+ * @tzobj: The timezone object, in a string.
+ *
+ * Sets the default timezone for the calendar, which is used to resolve
+ * DATE and floating DATE-TIME values.
+ */
+void
+e_cal_backend_set_default_zone (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *tzobj)
+{
+	g_return_if_fail (backend != NULL);
+	g_return_if_fail (E_IS_CAL_BACKEND (backend));
+	g_return_if_fail (tzobj != NULL);
+
+	(* CLASS (backend)->set_default_zone) (backend, cal, context, tzobj);
+}
+
+/**
+ * @deprecated This virual function should not be used in the backends, use
+ * e_cal_backend_set_zone instead. This function restricts the default timezone
+ * to be libical builtin timezone.
+ *
  * e_cal_backend_set_default_timezone:
  * @backend: A calendar backend.
  * @cal: An #EDataCal object.
  * @tzid: The TZID identifying the timezone.
- * 
+ *
  * Sets the default timezone for the calendar, which is used to resolve
- * DATE and floating DATE-TIME values. 
+ * DATE and floating DATE-TIME values.
+ *
  */
 void
-e_cal_backend_set_default_timezone (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *tzid)
+e_cal_backend_set_default_timezone (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *tzid)
 {
 	g_return_if_fail (backend != NULL);
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
 	g_return_if_fail (tzid != NULL);
 
-	g_assert (CLASS (backend)->set_default_timezone != NULL);
 	(* CLASS (backend)->set_default_timezone) (backend, cal, context, tzid);
 }
 
@@ -1007,7 +1030,7 @@ e_cal_backend_set_default_timezone (ECalBackend *backend, EDataCal *cal, DBusGMe
  * Add a timezone object to the given backend.
  */
 void
-e_cal_backend_add_timezone (ECalBackend *backend, EDataCal *cal, DBusGMethodInvocation *context, const char *tzobj)
+e_cal_backend_add_timezone (ECalBackend *backend, EDataCal *cal, EServerMethodContext context, const char *tzobj)
 {
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
 	g_return_if_fail (tzobj != NULL);
@@ -1062,7 +1085,7 @@ e_cal_backend_set_notification_proxy (ECalBackend *backend, ECalBackend *proxy)
 
 	g_return_if_fail (E_IS_CAL_BACKEND (backend));
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	priv->notification_proxy = proxy;
 }
@@ -1086,7 +1109,7 @@ e_cal_backend_notify_object_created (ECalBackend *backend, const char *calobj)
 	EIterator *iter;
 	EDataCalView *query;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	if (priv->notification_proxy) {
 		e_cal_backend_notify_object_created (priv->notification_proxy, calobj);
@@ -1100,7 +1123,7 @@ e_cal_backend_notify_object_created (ECalBackend *backend, const char *calobj)
 		query = QUERY (e_iterator_get (iter));
 
 		g_object_ref (query);
-		if (e_data_cal_view_object_matches (query, calobj))		
+		if (e_data_cal_view_object_matches (query, calobj))
 			e_data_cal_view_notify_objects_added_1 (query, calobj);
 		g_object_unref (query);
 
@@ -1112,25 +1135,48 @@ e_cal_backend_notify_object_created (ECalBackend *backend, const char *calobj)
 static void
 match_query_and_notify (EDataCalView *query, const char *old_object, const char *object)
 {
-	gboolean old_match, new_match;
+	gboolean old_match = FALSE, new_match = FALSE;
+
+	if (old_object)
+		old_match = e_data_cal_view_object_matches (query, old_object);
 
-	old_match = e_data_cal_view_object_matches (query, old_object);
 	new_match = e_data_cal_view_object_matches (query, object);
 	if (old_match && new_match)
 		e_data_cal_view_notify_objects_modified_1 (query, object);
 	else if (new_match)
 		e_data_cal_view_notify_objects_added_1 (query, object);
 	else if (old_match) {
-		icalcomponent *icalcomp;
-	
-		icalcomp = icalcomponent_new_from_string ((char *) old_object);
-		if (icalcomp) {
-			e_data_cal_view_notify_objects_removed_1 (query, icalcomponent_get_uid (icalcomp));
-			icalcomponent_free (icalcomp);
+		ECalComponent *comp = NULL;
+
+		comp = e_cal_component_new_from_string (old_object);
+		if (comp) {
+			ECalComponentId *id = e_cal_component_get_id (comp);
+
+			e_data_cal_view_notify_objects_removed_1 (query, id);
+
+			e_cal_component_free_id (id);
+			g_object_unref (comp);
 		}
 	}
 }
 
+/**
+ * e_cal_backend_notify_view_progress_start
+ * @backend: A calendar backend.
+ *
+ * This methods has to be used before e_cal_backend_notify_view_progress.
+ * Sets last notified percent value to 0.
+ **/
+void
+e_cal_backend_notify_view_progress_start (ECalBackend *backend)
+{
+	ECalBackendPrivate *priv;
+
+	priv = backend->priv;
+
+	priv->last_percent_notified = 0;
+}
+
 /**
  * e_cal_backend_notify_view_progress:
  * @backend: A calendar backend.
@@ -1147,7 +1193,12 @@ e_cal_backend_notify_view_progress (ECalBackend *backend, const char *message, i
 	EIterator *iter;
 	EDataCalView *query;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
+
+	if (percent <= priv->last_percent_notified)
+		return;
+
+	priv->last_percent_notified = percent;
 
 	if (priv->notification_proxy) {
 		e_cal_backend_notify_view_progress (priv->notification_proxy, message, percent);
@@ -1186,7 +1237,7 @@ e_cal_backend_notify_view_done (ECalBackend *backend, GNOME_Evolution_Calendar_C
 	EIterator *iter;
 	EDataCalView *query;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	if (priv->notification_proxy) {
 		e_cal_backend_notify_view_done (priv->notification_proxy, status);
@@ -1223,7 +1274,7 @@ e_cal_backend_notify_view_done (ECalBackend *backend, GNOME_Evolution_Calendar_C
  * modified by non-EDS clients.
  **/
 void
-e_cal_backend_notify_object_modified (ECalBackend *backend, 
+e_cal_backend_notify_object_modified (ECalBackend *backend,
 				      const char *old_object, const char *object)
 {
 	ECalBackendPrivate *priv;
@@ -1231,7 +1282,7 @@ e_cal_backend_notify_object_modified (ECalBackend *backend,
 	EIterator *iter;
 	EDataCalView *query;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	if (priv->notification_proxy) {
 		e_cal_backend_notify_object_modified (priv->notification_proxy, old_object, object);
@@ -1256,7 +1307,7 @@ e_cal_backend_notify_object_modified (ECalBackend *backend,
 /**
  * e_cal_backend_notify_object_removed:
  * @backend: A calendar backend.
- * @uid: the UID of the removed object
+ * @id: the Id of the removed object
  * @old_object: iCalendar representation of the removed object
  * @new_object: iCalendar representation of the object after the removal. This
  * only applies to recurrent appointments that had an instance removed. In that
@@ -1269,7 +1320,7 @@ e_cal_backend_notify_object_modified (ECalBackend *backend,
  * removed by non-EDS clients.
  **/
 void
-e_cal_backend_notify_object_removed (ECalBackend *backend, const char *uid,
+e_cal_backend_notify_object_removed (ECalBackend *backend, const ECalComponentId *id,
 				     const char *old_object, const char *object)
 {
 	ECalBackendPrivate *priv;
@@ -1277,10 +1328,10 @@ e_cal_backend_notify_object_removed (ECalBackend *backend, const char *uid,
 	EIterator *iter;
 	EDataCalView *query;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
+	priv = backend->priv;
 
 	if (priv->notification_proxy) {
-		e_cal_backend_notify_object_removed (priv->notification_proxy, uid, old_object, object);
+		e_cal_backend_notify_object_removed (priv->notification_proxy, id, old_object, object);
 		return;
 	}
 
@@ -1296,7 +1347,7 @@ e_cal_backend_notify_object_removed (ECalBackend *backend, const char *uid,
 			/* if object == NULL, it means the object has been completely
 			   removed from the backend */
 			if (e_data_cal_view_object_matches (query, old_object))
-				e_data_cal_view_notify_objects_removed_1 (query, uid);
+				e_data_cal_view_notify_objects_removed_1 (query, id);
 		} else
 			match_query_and_notify (query, old_object, object);
 
@@ -1307,6 +1358,25 @@ e_cal_backend_notify_object_removed (ECalBackend *backend, const char *uid,
 	g_object_unref (iter);
 }
 
+
+void
+e_cal_backend_notify_objects_added (ECalBackend *backend, EDataCalView *query, const GList *objects)
+{
+	e_data_cal_view_notify_objects_added (query, objects);
+}
+
+void
+e_cal_backend_notify_objects_removed (ECalBackend *backend, EDataCalView *query, const GList *ids)
+{
+	e_data_cal_view_notify_objects_removed (query, ids);
+}
+
+void
+e_cal_backend_notify_objects_modified (ECalBackend *backend, EDataCalView *query, const GList *objects)
+{
+	e_data_cal_view_notify_objects_modified (query, objects);
+}
+
 /**
  * e_cal_backend_notify_mode:
  * @backend: A calendar backend.
@@ -1318,14 +1388,12 @@ e_cal_backend_notify_object_removed (ECalBackend *backend, const char *uid,
  **/
 void
 e_cal_backend_notify_mode (ECalBackend *backend,
-			   EDataCalViewListenerSetModeStatus status, 
+			   EDataCalViewListenerSetModeStatus status,
 			   EDataCalMode mode)
 {
-	ECalBackendPrivate *priv;
+	ECalBackendPrivate *priv = backend->priv;
 	GList *l;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-
 	if (priv->notification_proxy) {
 		e_cal_backend_notify_mode (priv->notification_proxy, status, mode);
 		return;
@@ -1345,13 +1413,11 @@ e_cal_backend_notify_mode (ECalBackend *backend,
 void
 e_cal_backend_notify_auth_required (ECalBackend *backend)
 {
-	ECalBackendPrivate *priv;
-	GList *l;
-																																				  
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-	
-	for (l = priv->clients; l; l = l->next)
-		e_data_cal_notify_auth_required (l->data);
+        ECalBackendPrivate *priv = backend->priv;
+        GList *l;
+
+        for (l = priv->clients; l; l = l->next)
+                e_data_cal_notify_auth_required (l->data);
 }
 
 /**
@@ -1364,11 +1430,9 @@ e_cal_backend_notify_auth_required (ECalBackend *backend)
 void
 e_cal_backend_notify_error (ECalBackend *backend, const char *message)
 {
-	ECalBackendPrivate *priv;
+	ECalBackendPrivate *priv = backend->priv;
 	GList *l;
 
-	priv = E_CAL_BACKEND_GET_PRIVATE (backend);
-
 	if (priv->notification_proxy) {
 		e_cal_backend_notify_error (priv->notification_proxy, message);
 		return;
