@@ -2,7 +2,7 @@
 /*
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *
- *  Copyright 1999-2003 Ximian, Inc. (www.ximian.com)
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -24,35 +24,37 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
-#include <limits.h>
-#include <sys/types.h>
-#include <dirent.h>
-#include <sys/stat.h>
-#include <unistd.h>
 #include <errno.h>
-#include <string.h>
 #include <fcntl.h>
+#include <limits.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
-#ifndef _POSIX_PATH_MAX
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
+
+#if !defined (G_OS_WIN32) && !defined (_POSIX_PATH_MAX)
 #include <posix1_lim.h>
 #endif
 
-#include "camel-local-folder.h"
-#include "camel-local-store.h"
-#include "camel-stream-fs.h"
-#include "camel-local-summary.h"
 #include "camel-data-wrapper.h"
-#include "camel-mime-message.h"
-#include "camel-stream-filter.h"
-#include "camel-mime-filter-from.h"
 #include "camel-exception.h"
-#include "camel-i18n.h"
-
-#include "camel-local-private.h"
-
+#include "camel-mime-filter-from.h"
+#include "camel-mime-message.h"
+#include "camel-private.h"
+#include "camel-stream-filter.h"
+#include "camel-stream-fs.h"
 #include "camel-text-index.h"
 
+#include "camel-local-folder.h"
+#include "camel-local-private.h"
+#include "camel-local-store.h"
+#include "camel-local-summary.h"
+
 #define d(x) /*(printf("%s(%d): ", __FILE__, __LINE__),(x))*/
 
 #ifndef PATH_MAX
@@ -67,10 +69,10 @@ static GSList *local_folder_properties;
 #define CF_CLASS(so) CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 #define CLOCALS_CLASS(so) CAMEL_STORE_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 
-static int local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args);
-static int local_setv(CamelObject *object, CamelException *ex, CamelArgV *args);
+static gint local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args);
+static gint local_setv(CamelObject *object, CamelException *ex, CamelArgV *args);
 
-static int local_lock(CamelLocalFolder *lf, CamelLockType type, CamelException *ex);
+static gint local_lock(CamelLocalFolder *lf, CamelLockType type, CamelException *ex);
 static void local_unlock(CamelLocalFolder *lf);
 
 static void local_refresh_info(CamelFolder *folder, CamelException *ex);
@@ -78,12 +80,14 @@ static void local_refresh_info(CamelFolder *folder, CamelException *ex);
 static void local_sync(CamelFolder *folder, gboolean expunge, CamelException *ex);
 static void local_expunge(CamelFolder *folder, CamelException *ex);
 
-static GPtrArray *local_search_by_expression(CamelFolder *folder, const char *expression, CamelException *ex);
-static GPtrArray *local_search_by_uids(CamelFolder *folder, const char *expression, GPtrArray *uids, CamelException *ex);
+static GPtrArray *local_search_by_expression(CamelFolder *folder, const gchar *expression, CamelException *ex);
+static guint32 local_count_by_expression(CamelFolder *folder, const gchar *expression, CamelException *ex);
+static GPtrArray *local_search_by_uids(CamelFolder *folder, const gchar *expression, GPtrArray *uids, CamelException *ex);
 static void local_search_free(CamelFolder *folder, GPtrArray * result);
+static GPtrArray * local_get_uncached_uids (CamelFolder *folder, GPtrArray * uids, CamelException *ex);
 
 static void local_delete(CamelFolder *folder);
-static void local_rename(CamelFolder *folder, const char *newname);
+static void local_rename(CamelFolder *folder, const gchar *newname);
 
 static void local_finalize(CamelObject * object);
 
@@ -102,8 +106,10 @@ camel_local_folder_class_init(CamelLocalFolderClass * camel_local_folder_class)
 	camel_folder_class->refresh_info = local_refresh_info;
 	camel_folder_class->sync = local_sync;
 	camel_folder_class->expunge = local_expunge;
+	camel_folder_class->get_uncached_uids = local_get_uncached_uids;
 
 	camel_folder_class->search_by_expression = local_search_by_expression;
+	camel_folder_class->count_by_expression = local_count_by_expression;
 	camel_folder_class->search_by_uids = local_search_by_uids;
 	camel_folder_class->search_free = local_search_free;
 
@@ -147,9 +153,8 @@ local_finalize(CamelObject * object)
 		folder->summary = NULL;
 	}
 
-	if (local_folder->search) {
+	if (local_folder->search)
 		camel_object_unref((CamelObject *)local_folder->search);
-	}
 
 	if (local_folder->index)
 		camel_object_unref((CamelObject *)local_folder->index);
@@ -163,9 +168,9 @@ local_finalize(CamelObject * object)
 	g_free(local_folder->index_path);
 
 	camel_folder_change_info_free(local_folder->changes);
-	
+
 	g_mutex_free(local_folder->priv->search_lock);
-	
+
 	g_free(local_folder->priv);
 }
 
@@ -179,7 +184,7 @@ camel_local_folder_get_type(void)
 	static CamelType camel_local_folder_type = CAMEL_INVALID_TYPE;
 
 	if (camel_local_folder_type == CAMEL_INVALID_TYPE) {
-		int i;
+		gint i;
 
 		parent_class = (CamelFolderClass *)camel_folder_get_type();
 		camel_local_folder_type = camel_type_register(camel_folder_get_type(), "CamelLocalFolder",
@@ -200,25 +205,24 @@ camel_local_folder_get_type(void)
 }
 
 CamelLocalFolder *
-camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, const char *full_name, guint32 flags, CamelException *ex)
+camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, const gchar *full_name, guint32 flags, CamelException *ex)
 {
 	CamelFolderInfo *fi;
 	CamelFolder *folder;
-	const char *root_dir_path, *name;
-	char *tmp, *statepath;
-	char folder_path[PATH_MAX];
+	const gchar *root_dir_path;
+	gchar *name;
+	gchar *tmp, *statepath;
+#ifndef G_OS_WIN32
+	gchar folder_path[PATH_MAX];
 	struct stat st;
-	int forceindex, len;
+#endif
+	gint forceindex, len;
 	CamelURL *url;
 	CamelLocalStore *ls = (CamelLocalStore *)parent_store;
-	
+
 	folder = (CamelFolder *)lf;
 
-	name = strrchr(full_name, '/');
-	if (name)
-		name++;
-	else
-		name = full_name;
+	name = g_path_get_basename(full_name);
 
 	camel_folder_construct(folder, parent_store, full_name, name);
 
@@ -227,7 +231,7 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	len = strlen (root_dir_path);
 	tmp = g_alloca (len + 1);
 	strcpy (tmp, root_dir_path);
-	if (len>1 && tmp[len-1] == '/')
+	if (len>1 && G_IS_DIR_SEPARATOR(tmp[len-1]))
 		tmp[len-1] = 0;
 
 	lf->base_path = g_strdup(root_dir_path);
@@ -247,26 +251,26 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 		camel_object_set(lf, NULL, CAMEL_LOCAL_FOLDER_INDEX_BODY, TRUE, 0);
 		camel_object_state_write(lf);
 	}
-
+#ifndef G_OS_WIN32
 	/* follow any symlinks to the mailbox */
 	if (lstat (lf->folder_path, &st) != -1 && S_ISLNK (st.st_mode) &&
 	    realpath (lf->folder_path, folder_path) != NULL) {
 		g_free (lf->folder_path);
 		lf->folder_path = g_strdup (folder_path);
 	}
-	
+#endif
 	lf->changes = camel_folder_change_info_new();
 
 	/* TODO: Remove the following line, it is a temporary workaround to remove
 	   the old-format 'ibex' files that might be lying around */
-	unlink(lf->index_path);
+	g_unlink(lf->index_path);
 
 	/* FIXME: Need to run indexing off of the setv method */
 
 	/* if we have no/invalid index file, force it */
 	forceindex = camel_text_index_check(lf->index_path) == -1;
 	if (lf->flags & CAMEL_STORE_FOLDER_BODY_INDEX) {
-		int flag = O_RDWR|O_CREAT;
+		gint flag = O_RDWR|O_CREAT;
 
 		if (forceindex)
 			flag |= O_TRUNC;
@@ -274,7 +278,7 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 		lf->index = (CamelIndex *)camel_text_index_new(lf->index_path, flag);
 		if (lf->index == NULL) {
 			/* yes, this isn't fatal at all */
-			g_warning("Could not open/create index file: %s: indexing not performed", strerror (errno));
+			g_warning("Could not open/create index file: %s: indexing not performed", g_strerror (errno));
 			forceindex = FALSE;
 			/* record that we dont have an index afterall */
 			lf->flags &= ~CAMEL_STORE_FOLDER_BODY_INDEX;
@@ -287,8 +291,16 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	}
 
 	folder->summary = (CamelFolderSummary *)CLOCALF_CLASS(lf)->create_summary(lf, lf->summary_path, lf->folder_path, lf->index);
-	if (camel_local_summary_load((CamelLocalSummary *)folder->summary, forceindex, NULL) == -1) {
+	if (!(flags & CAMEL_STORE_IS_MIGRATING) && camel_local_summary_load((CamelLocalSummary *)folder->summary, forceindex, NULL) == -1) {
 		/* ? */
+		if (camel_local_summary_check((CamelLocalSummary *)folder->summary, lf->changes, ex) == 0) {
+			/* we sync here so that any hard work setting up the folder isn't lost */
+			if (camel_local_summary_sync((CamelLocalSummary *)folder->summary, FALSE, lf->changes, ex) == -1) {
+				camel_object_unref (CAMEL_OBJECT (folder));
+				g_free(name);
+				return NULL;
+			}
+		}
 	}
 
 	/* We don't need to sync here ..., it can sync later on when it calls refresh info */
@@ -297,6 +309,7 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	/* we sync here so that any hard work setting up the folder isn't lost */
 	/*if (camel_local_summary_sync((CamelLocalSummary *)folder->summary, FALSE, lf->changes, ex) == -1) {
 		camel_object_unref (CAMEL_OBJECT (folder));
+		g_free(name);
 		return NULL;
 		}*/
 #endif
@@ -305,26 +318,26 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	if ((flags & CAMEL_STORE_FOLDER_CREATE) != 0) {
 		url = camel_url_copy (((CamelService *) parent_store)->url);
 		camel_url_set_fragment (url, full_name);
-	
-		fi = g_new0 (CamelFolderInfo, 1);
+
+		fi = camel_folder_info_new ();
 		fi->full_name = g_strdup (full_name);
 		fi->name = g_strdup (name);
 		fi->uri = camel_url_to_string (url, 0);
 		fi->unread = camel_folder_get_unread_message_count(folder);
 		fi->flags = CAMEL_FOLDER_NOCHILDREN;
-	
+
 		camel_url_free (url);
-	
+
 		camel_object_trigger_event(CAMEL_OBJECT (parent_store), "folder_created", fi);
 		camel_folder_info_free(fi);
 	}
-	
+	g_free(name);
 	return lf;
 }
 
 /* lock the folder, may be called repeatedly (with matching unlock calls),
    with type the same or less than the first call */
-int camel_local_folder_lock(CamelLocalFolder *lf, CamelLockType type, CamelException *ex)
+gint camel_local_folder_lock(CamelLocalFolder *lf, CamelLockType type, CamelException *ex)
 {
 	if (lf->locked > 0) {
 		/* lets be anal here - its important the code knows what its doing */
@@ -341,7 +354,7 @@ int camel_local_folder_lock(CamelLocalFolder *lf, CamelLockType type, CamelExcep
 }
 
 /* unlock folder */
-int camel_local_folder_unlock(CamelLocalFolder *lf)
+gint camel_local_folder_unlock(CamelLocalFolder *lf)
 {
 	g_assert(lf->locked>0);
 	lf->locked--;
@@ -351,11 +364,11 @@ int camel_local_folder_unlock(CamelLocalFolder *lf)
 	return 0;
 }
 
-static int
+static gint
 local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 {
 	CamelFolder *folder = (CamelFolder *)object;
-	int i;
+	gint i;
 	guint32 tag;
 
 	for (i=0;i<args->argc;i++) {
@@ -366,7 +379,8 @@ local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 		switch (tag & CAMEL_ARG_TAG) {
 		case CAMEL_OBJECT_ARG_DESCRIPTION:
 			if (folder->description == NULL) {
-				char *tmp, *path;
+				const gchar *tmp;
+				gchar *path;
 
 				/* check some common prefixes to shorten the name */
 				tmp = ((CamelService *)folder->parent_store)->url->path;
@@ -382,14 +396,14 @@ local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 									      ((CamelService *)folder->parent_store)->url->protocol);
 				else if ((tmp = "/var/spool/mail") && strncmp(tmp, path, strlen(tmp)) == 0)
 					/* /var/spool/mail relative path + protocol */
-					folder->description = g_strdup_printf(_("mailbox:%s (%s)"), path+strlen(tmp),
+					folder->description = g_strdup_printf(_("mailbox: %s (%s)"), path+strlen(tmp),
 									      ((CamelService *)folder->parent_store)->url->protocol);
 				else if ((tmp = "/var/mail") && strncmp(tmp, path, strlen(tmp)) == 0)
-					folder->description = g_strdup_printf(_("mailbox:%s (%s)"), path+strlen(tmp),
+					folder->description = g_strdup_printf(_("mailbox: %s (%s)"), path+strlen(tmp),
 									      ((CamelService *)folder->parent_store)->url->protocol);
 				else
 					/* a full path + protocol */
-					folder->description = g_strdup_printf(_("%s (%s)"), path, 
+					folder->description = g_strdup_printf(_("%s (%s)"), path,
 									      ((CamelService *)folder->parent_store)->url->protocol);
 			}
 			*arg->ca_str = folder->description;
@@ -421,10 +435,10 @@ local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 	return ((CamelObjectClass *)parent_class)->getv(object, ex, args);
 }
 
-static int
+static gint
 local_setv(CamelObject *object, CamelException *ex, CamelArgV *args)
 {
-	int i;
+	gint i;
 	guint32 tag;
 
 	for (i=0;i<args->argc;i++) {
@@ -452,7 +466,7 @@ local_setv(CamelObject *object, CamelException *ex, CamelArgV *args)
 	return ((CamelObjectClass *)parent_class)->setv(object, ex, args);
 }
 
-static int
+static gint
 local_lock(CamelLocalFolder *lf, CamelLockType type, CamelException *ex)
 {
 	return 0;
@@ -470,13 +484,32 @@ local_refresh_info(CamelFolder *folder, CamelException *ex)
 {
 	CamelLocalFolder *lf = (CamelLocalFolder *)folder;
 
-	if (camel_local_summary_check((CamelLocalSummary *)folder->summary, lf->changes, ex) == -1)
-		return;
+	/*
+	 * Banner: This is a very very ugly hack to get over the summary mismatch. This needs to
+	 * be done better. Im postponing this post-disk summary.
+	 * */
 
+	CAMEL_FOLDER_REC_LOCK(folder, lock);
+
+	if (camel_local_summary_check((CamelLocalSummary *)folder->summary, lf->changes, ex) == -1) {
+		CAMEL_FOLDER_REC_UNLOCK(folder, lock);
+		return;
+	}
+
+	CAMEL_FOLDER_REC_UNLOCK(folder, lock);
 	if (camel_folder_change_info_changed(lf->changes)) {
 		camel_object_trigger_event((CamelObject *)folder, "folder_changed", lf->changes);
 		camel_folder_change_info_clear(lf->changes);
 	}
+
+}
+
+static GPtrArray *
+local_get_uncached_uids (CamelFolder *folder, GPtrArray * uids, CamelException *ex)
+{
+	GPtrArray *result = g_ptr_array_new ();
+	/* By default, we would have everything local. No need to fetch from anywhere. */
+	return result;
 }
 
 static void
@@ -515,7 +548,7 @@ static void
 local_delete(CamelFolder *folder)
 {
 	CamelLocalFolder *lf = (CamelLocalFolder *)folder;
-	
+
 	if (lf->index)
 		camel_index_delete(lf->index);
 
@@ -523,10 +556,10 @@ local_delete(CamelFolder *folder)
 }
 
 static void
-local_rename(CamelFolder *folder, const char *newname)
+local_rename(CamelFolder *folder, const gchar *newname)
 {
 	CamelLocalFolder *lf = (CamelLocalFolder *)folder;
-	char *statepath;
+	gchar *statepath;
 	CamelLocalStore *ls = (CamelLocalStore *)folder->parent_store;
 
 	d(printf("renaming local folder paths to '%s'\n", newname));
@@ -553,7 +586,7 @@ local_rename(CamelFolder *folder, const char *newname)
 }
 
 static GPtrArray *
-local_search_by_expression(CamelFolder *folder, const char *expression, CamelException *ex)
+local_search_by_expression(CamelFolder *folder, const gchar *expression, CamelException *ex)
 {
 	CamelLocalFolder *local_folder = CAMEL_LOCAL_FOLDER(folder);
 	GPtrArray *matches;
@@ -572,8 +605,28 @@ local_search_by_expression(CamelFolder *folder, const char *expression, CamelExc
 	return matches;
 }
 
+static guint32
+local_count_by_expression(CamelFolder *folder, const gchar *expression, CamelException *ex)
+{
+	CamelLocalFolder *local_folder = CAMEL_LOCAL_FOLDER(folder);
+	gint matches;
+
+	CAMEL_LOCAL_FOLDER_LOCK(folder, search_lock);
+
+	if (local_folder->search == NULL)
+		local_folder->search = camel_folder_search_new();
+
+	camel_folder_search_set_folder(local_folder->search, folder);
+	camel_folder_search_set_body_index(local_folder->search, local_folder->index);
+	matches = camel_folder_search_count (local_folder->search, expression, ex);
+
+	CAMEL_LOCAL_FOLDER_UNLOCK(folder, search_lock);
+
+	return matches;
+}
+
 static GPtrArray *
-local_search_by_uids(CamelFolder *folder, const char *expression, GPtrArray *uids, CamelException *ex)
+local_search_by_uids(CamelFolder *folder, const gchar *expression, GPtrArray *uids, CamelException *ex)
 {
 	CamelLocalFolder *local_folder = CAMEL_LOCAL_FOLDER(folder);
 	GPtrArray *matches;
